pub struct SquashedOffset(pub u8);Expand description
Indicates which version of a Table is referred to by a RowPointer.
Currently, SquashedOffset has two meaningful values,
SquashedOffset::TX_STATE and SquashedOffset::COMMITTED_STATE,
which refer to the TX scratchpad and the committed state respectively.
In the future, SquashedOffset will be extended to capture
which savepoint within a transaction the pointer refers to,
or which committed-unsquashed preceding transaction.
Tuple Fields
0: u8Implementations
Sourceimpl SquashedOffset
impl SquashedOffset
Sourcepub const COMMITTED_STATE: Self
pub const COMMITTED_STATE: Self
The SquashedOffset for the committed (squashed) state.
Sourcepub const fn is_tx_state(self) -> bool
pub const fn is_tx_state(self) -> bool
Does this SquahsedOffset refer to the TX scratchpad?
Sourcepub const fn is_committed_state(self) -> bool
pub const fn is_committed_state(self) -> bool
Does this SquashedOffset refer to the committed (squashed) state?