pub type Byte = u8;
A byte is a u8.
u8
Previous implementations used MaybeUninit<u8> here, but it became necessary to serialize pages to enable snapshotting, so we require that all bytes in a page be valid u8s, never uninit.
MaybeUninit<u8>