FixedView

Struct FixedView 

pub struct FixedView<'page> { /* private fields */ }
A mutable view of the fixed-len section of a Page.

Implementations

Source

impl FixedView<'_>

pub fn get_row_mut( &mut self, start: PageOffset, fixed_row_size: Size, ) -> &mut Bytes

Returns a mutable view of the row from start lasting fixed_row_size number of bytes.

This method is safe, but callers should take care that start and fixed_row_size are correct for this page, and that start is aligned. Callers should further ensure that mutations to the row leave the row bytes in an expected state, i.e. initialized where required by the row type, and with VarLenRefs that point to valid granules and with correct lengths.

pub unsafe fn free(&mut self, row_offset: PageOffset, fixed_row_size: Size)

Frees the row starting at row_offset and lasting fixed_row_size bytes.

Safety

range_move(0..fixed_row_size, row_offset) must be in bounds of row_data. Moreover, it must be valid for writing a FreeCellRef to it, which includes being properly aligned with respect to row_data for a FreeCellRef.