AlignedVarLenOffsets

Struct AlignedVarLenOffsets 

pub struct AlignedVarLenOffsets<'a>(/* private fields */);
Slice of offsets to var-len members, in units of 2-byte words.

This type is intended as a demonstration of the VarLenMembers interface, and is used in testing and benchmarking.

Note that this visitor is not suitable for sum types, or for types which contain sums.

Units of 2-byte words because VarLenRef is 2-byte aligned. Note that VarLenRef is 4 bytes wide, but only 2-byte aligned.

The listed offsets must not overlap, i.e. there must be a gap of at least 2 between each offset.

For AlignedVarLenOffsets([n]), a 4-byte VarLenRef exists in each row at +2n bytes.

e.g.: AlignedVarLenOffsets([0, 4]) has: row >= 12 bytes,

  • var-len ref at +0..4 bytes (i.e. +0..2 u16s).
  • fixed-len field(s) at +4..8 bytes (i.e. +2..4 u16s).
  • var-len ref at +8..12 bytes (i.e. +4..6 u16s).
  • fixed-len field(s) at +12.. (i.e. +6.. u16s), if row_size > 12.

Implementations

Source

impl<'a> AlignedVarLenOffsets<'a>

pub const fn from_offsets(offsets: &'a [u16]) -> Self

Returns an AlignedVarLenOffsets using offsets.